#-*- coding: utf-8 -*-

import sys
import os
import argparse
import configparser
try:
    from l2lss.dcvsapi import Repo
    from l2lss.utils import get_dcvs_type, options2sections, sections_list
except ImportError:
    from dcvsapi import Repo
    from utils import get_dcvs_type, options2sections, sections_list


def init(*sys_args):
    """Initialise the repo.

    Create directories and config file.
    Save the config file.
    """
    args = parse(*sys_args)
    check(args)
    if not os.path.exists(args.path):
        os.mkdir(args.path)
    if not args.no_init_code_repo and os.listdir(args.path):
        print('Directory not empty. Cannot initialize.\nExiting.')
        sys.exit(1)
    os.chdir(args.path)
    # Initialise repo
    os.mkdir('.l2lss')
    save_config(args)
    if not args.no_init_code_repo:
        repo_type = 'hg' if args.hg else 'git'
        repo = Repo(type=repo_type)
        repo.init()
        update_ignore_file(repo)


def parse(*sys_args):
    """Parse the args from sys.args and returns the result."""
    parser = argparse.ArgumentParser(description='''Initialise the local2local_repo. If no repository type is defined, we will use git.
    All commands are run where the script is.''')
    parser.add_argument('--bare',  dest='bare',  action='store_true',
                            help='Initialise the server side repo')
    parser.add_argument('--path',  '-p',  dest='path',  default='.',
                            help='The path were to initialize the local2local_repo.')
    parser.add_argument('--git', dest='git', action='store_true',
                            help='Use a git repo for code.')
    parser.add_argument('--hg',  '--mercurial',  dest='hg',  action='store_true',
                            help='Use a mercurial repo for code.')
    parser.add_argument('--no-init-code-repo',  '-n',  dest='no_init_code_repo',  action='store_true',
                            help='Skip code repo initialization. A repository must already exits.')
    parser.add_argument('--sync-path', '-s', dest='sync_path',
                            help='Where to synchronise the data.')
    parser.add_argument('--dump-cmd', '-d',  dest='dump_cmd',
                            help='The command we should use to dump the database.')
    parser.add_argument('--restore-cmd', '-r', dest='restore_cmd',
                            help='The command we should use to restore the database.')
    parser.add_argument('--compress-db',  '-c', dest='compress',
                            help='Compress the database dump. Argument must be gzip, xz or bzip2.')
    parser.add_argument('--files-dir', '-f',  dest='files_dir',
                            help='The relative path from the repo root to the files that must be sync.')
    parser.add_argument('--encoding', '-e', dest='encoding',
                            help='The encoding of the database file.')
    args = parser.parse_args()

    return args


def check(args):
    """Performs various check in order to avoid incoherence."""
    if args.git and args.hg:
        print('You cannot use git and mercurial for code repository. Please choose.\nExiting.')
        sys.exit(1)
    if args.no_init_code_repo and (args.hg or args.hg):
        print('Ask not to init repo and yet asking to init it.\nExiting.')
        sys.exit(1)
    if args.compress and args.compress not in ['xz', 'gzip', 'bzip2']:
        print('The compression argument is not valid.')
        sys.exit(1)


def save_config(args):
    """Save the config files.

    For all available options that need storing, save them if they are provided.
    """
    config = get_empty_config()

    for option, section in options2sections.items():
        raw_option = option.replace(' ', '_')
        args_option = getattr(args, raw_option)
        if args_option:
            config[section][option] = args_option

    with open('.l2lss/config', 'w') as config_file:
        config.write(config_file)


def get_empty_config():
    config = configparser.ConfigParser()
    for section in sections_list:
        config[section] = {}
    return config


def update_ignore_file(repo):
    # Update .gitignore or .hgignore file
    dcvs_type = get_dcvs_type()
    if dcvs_type:
        ignore_file_name = '.{}ignore'.format(dcvs_type)
        with open(ignore_file_name, 'a') as ignore_file:
            ignore_file.write('.l2lss')
        repo.add(ignore_file_name)
        repo.commit("Ignore l2lss files")
